;==============================================================================
;*Author:  Jonathan Weaver, jonw0224@netscape.net
;E-mail Contact: jonw0224@netscape.net
;*Description:  Reads and writes to EEPROM.  Tested with PIC16F84A and PIC16F877A.  Should offer, as a minimum, compatibility with PIC16F84(A) and PIC16F87X(A) microcontrollers.
;*Version: 1.0
;*Date: 11/9/2005
;*Filename: eeprom.inc
;
;Program Words:	29
;Data Words:	 1
;
;Versions:  1.0 - 1/2/2003 -- Code finished, tested, and cleaned.
;			1.0 - 9/29/2005 -- Comments and license added
;			1.0 - 11/9/2005 -- Generalized banking
;
;Copyright (C) 2005 Jonathan Weaver
;
;This program is free software; you can redistribute it and/or modify it under
;the terms of the GNU General Public License as published by the Free Software
;Foundation; either version 2 of the License, or (at your option) any later
;version.
;
;This program is distributed in the hope that it will be useful, but WITHOUT
;ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;details.
;
;You should have received a copy of the GNU General Public License along with
;this program; if not, write to the Free Software Foundation, Inc.,
;51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
;
;==============================================================================


;;;;;;; Variables ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;Declare outside of the include file
;			eeadrw		;Temporary address location used by EEPROM_write

;;;;;; EEPROM_read subroutine ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;*Read data from EEPROM.  Expects:  EEPROM address in WREG.  Uses BNKSEL from banks.inc.
;*WREG:  address to read from
;*Returns:  EEPROM data in WREG
EEPROM_read		;public
		BNKSEL EECON1
EEPROM_readL1
		btfsc EECON1, WR		;if WR is set, wait
			goto EEPROM_readL1
		BNKSEL EEADR
		movwf EEADR				;Address to read
		BNKSEL EECON1
		ifdef EEPGD
			bcf EECON1, EEPGD
		endif
		bsf EECON1, RD			;EE Read
		BNKSEL EEDATA
		movf EEDATA, W
		BNKSEL 0x0000			;Go to bank zero
		return

;;;;;; EEPROM_write subroutine ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;*Write data to EEPROM Expects:  DATA to write in WREG and ADDRESS to write in eeadrw.  Uses BNKSEL from banks.inc.
;*WREG:  Data to write
;*eeadrw:  address to write to, should be accessable in same bank as EEADR.
;*Returns:  Nothing
EEPROM_write:	;public
		BNKSEL EECON1
EEPROM_writeL1
		btfsc EECON1, WR		;if WR is set, wait
			goto EEPROM_writeL1
		BNKSEL EEDATA
		movwf EEDATA			;move WREG to EEDATA
		movf eeadrw, W
		movwf EEADR				;move eeadrw to EEADR
		BNKSEL EECON1
		bcf INTCON, GIE			;Disable interrupts
		bsf EECON1, WREN		;Enable EEPROM write
		movlw 0x55
		movwf EECON2			;Magic combination on EECON2
		movlw 0xAA
		movwf EECON2
		bsf EECON1, WR			;begin write
		bsf INTCON, GIE			;Enable interrupts
		bcf EECON1, WREN		;Disable EEPROM write
		BNKSEL 0x0000			;Go to bank zero
		return
